#!/bin/bash
#
# Shell script to check the md5sum of the libs
# "before" AND "after" a/some change(s), if there are any differences
#
# Copyright 2014 by guy maurel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# 12 january 2014
#
numberOfParameter=$#
if [ $numberOfParameter -eq 0 ] ;
then
  echo "Use the script with one command"
  echo "  Command can be:"
  echo "    save:  save the md5sum before any change"
  echo "    test:  test all the saved md5sum again the new one"
  echo "    clean: remove all the saved md5sum and the save-directory"
  exit 0
fi
#
CMD=$1
if [ $CMD == "save" ] ;
then
  command="1"
else
  if [ $CMD == "test" ] ;
  then
    command="2"
  else
    if [ $CMD == "clean" ] ;
    then
      command="3"
    else
      echo "command" $CMD "not known"
      exit 0
    fi
  fi
fi
#
# check we are at "build"
WHERE_ARE_WE=`pwd`
#
if [ $CMD == "clean" ] ;
then
  rm -rf save
  echo "all the saved md5sum and the save-directory are removed"
  exit 1
fi
if [ $CMD == "save" ] ;
then
  # make the "save"-directory
  mkdir -p save
else
  if [[ ! -d save ]] ;
  then
    echo "the save-directory is not present"
    echo "run the script with the save-command before"
    echo "  Md5sum-the-Libs.sh save"
    exit 0
  fi
fi
#
if [ $CMD == "save" ] ;
then
  # make a list of the md5sum to be saved
  find . -type f -name *.so* > save/to_be_saved.list
  LIST=`cat save/to_be_saved.list`
  for lib_file in $LIST
  do
    echo "save the md5sum "$lib_file
    # save the md5sum
    WHERE_TO_SAVE=`dirname $lib_file`
    mkdir -p save/$WHERE_TO_SAVE
    md5sum $lib_file > save/$lib_file.md5sum
    cp $lib_file save/$lib_file
  done
  touch save/time_stamp.txt
  echo "all md5sum are saved"
  exit 1
else
  # the command is "test" it
  # look for the new md5sum. They are newer as save/time_stamp.txt
  #find . -newer save/time_stamp.txt -name *.so > save/new_md5sum.list
  find . -type f -name *.so* | grep -v "/save/" > save/new_md5sum.list
  LIST=`cat save/new_md5sum.list`
  for lib_file in $LIST
  do
    echo "test the md5sum for "$lib_file
    md5sum --check --status save/$lib_file.md5sum > /dev/null 2>&1
    how_different=$?
    if [ $how_different != "0" ] ;
    then
      echo "the status of md5sum --check --status is "$how_different
      echo "for "$lib_file
    fi
    #
    echo "test the "$lib_file
    diff ${lib_file} save/${lib_file} >/dev/null 2>&1
    how_different=$?
    if [ $how_different != "0" ] ;
    then
      echo "the status of diff is "$how_different
      echo "for "$lib_file
      echo "diff ${lib_file} save/${lib_file}"
      ls -l ${lib_file} save/${lib_file}
      exit 0
    fi
  done
  rm save/new_md5sum.list
  echo "all tests are OK"
  exit 1
fi
